@@ -24,9 +24,11 @@ import org.codehaus.groovy.ast.expr.Expression;
 import org.codehaus.groovy.ast.expr.FieldExpression;
 import org.codehaus.groovy.ast.expr.MapExpression;
 import org.codehaus.groovy.ast.expr.TupleExpression;
+import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.ast.stmt.ExpressionStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
-import org.codehaus.groovy.ast.stmt.BlockStatement;
+import org.codehaus.groovy.ast.tools.ClassNodeUtils;
+import org.codehaus.groovy.ast.tools.ParameterUtils;
 import org.codehaus.groovy.control.CompilePhase;
 import org.codehaus.groovy.transform.ASTTransformation;
 import org.codehaus.groovy.transform.GroovyASTTransformation;
@@ -34,8 +36,17 @@ import org.codehaus.groovy.vmplugin.VMPluginFactory;
 import org.objectweb.asm.Opcodes;
 
 import java.lang.reflect.Array;
-import java.lang.reflect.Constructor;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.EnumMap;
+import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.ListIterator;
+import java.util.Map;
+import java.util.Set;
 
 /**
  * Represents a class in the AST.
@@ -92,21 +103,24 @@ import java.util.*;
  * by the class are needed, then a call to {@link #redirect()} will help.
  *
  * @see org.codehaus.groovy.ast.ClassHelper
- * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @author Jochen Theodorou
  */
 public class ClassNode extends AnnotatedNode implements Opcodes {
     private static class MapOfLists {
-        private Map<Object, List<MethodNode>> map = new HashMap<Object, List<MethodNode>>();
+        private Map<Object, List<MethodNode>> map;
         public List<MethodNode> get(Object key) {
-            return map.get(key);
+            return map == null ? null : map.get(key);
         }
+
         public List<MethodNode> getNotNull(Object key) {
             List<MethodNode> ret = get(key);
             if (ret==null) ret = Collections.emptyList();
             return ret;
         }
+
         public void put(Object key, MethodNode value) {
+            if (map == null) {
+                 map = new LinkedHashMap<Object, List<MethodNode>>();
+            }
             if (map.containsKey(key)) {
                 get(key).add(value);
             } else {
@@ -115,6 +129,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
                 map.put(key, list);
             }
         }
+
         public void remove(Object key, MethodNode value) {
             get(key).remove(value);
         }
@@ -411,7 +426,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     public Set<ClassNode> getAllInterfaces () {
-        Set<ClassNode> res = new HashSet<ClassNode>();
+        Set<ClassNode> res = new LinkedHashSet<ClassNode>();
         getAllInterfaces(res);
         return res;
     }
@@ -429,23 +444,13 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     public Map<String, MethodNode> getDeclaredMethodsMap() {
         // Start off with the methods from the superclass.
         ClassNode parent = getSuperClass();
-        Map<String, MethodNode> result = null;
+        Map<String, MethodNode> result;
         if (parent != null) {
             result = parent.getDeclaredMethodsMap();
         } else {
-            result = new HashMap<String, MethodNode>();
-        }
-
-        // add in unimplemented abstract methods from the interfaces
-        for (ClassNode iface : getInterfaces()) {
-            Map<String, MethodNode> ifaceMethodsMap = iface.getDeclaredMethodsMap();
-            for (String methSig : ifaceMethodsMap.keySet()) {
-                if (!result.containsKey(methSig)) {
-                    MethodNode methNode = ifaceMethodsMap.get(methSig);
-                    result.put(methSig, methNode);
-                }
-            }
+            result = new LinkedHashMap<String, MethodNode>();
         }
+        ClassNodeUtils.addInterfaceMethods(this, result);
 
         // And add in the methods implemented in this class.
         for (MethodNode method : getMethods()) {
@@ -530,7 +535,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         if (r.fields == null)
             r.fields = new LinkedList<FieldNode> ();
         if (r.fieldIndex == null)
-            r.fieldIndex = new HashMap<String,FieldNode> ();
+            r.fieldIndex = new LinkedHashMap<String,FieldNode> ();
         r.fields.add(node);
         r.fieldIndex.put(node.getName(), node);
     }
@@ -542,7 +547,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         if (r.fields == null)
             r.fields = new LinkedList<FieldNode> ();
         if (r.fieldIndex == null)
-            r.fieldIndex = new HashMap<String,FieldNode> ();
+            r.fieldIndex = new LinkedHashMap<String,FieldNode> ();
         r.fields.addFirst(node);
         r.fieldIndex.put(node.getName(), node);
     }
@@ -742,7 +747,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         if (!redirect().lazyInitDone) redirect().lazyClassInit();
         ClassNode r = redirect ();
         if (r.fieldIndex == null)
-            r.fieldIndex = new HashMap<String,FieldNode> ();
+            r.fieldIndex = new LinkedHashMap<String,FieldNode> ();
         return r.fieldIndex.get(name);
     }
 
@@ -1036,17 +1041,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @return true if the two arrays are of the same size and have the same contents
      */
     protected boolean parametersEqual(Parameter[] a, Parameter[] b) {
-        if (a.length == b.length) {
-            boolean answer = true;
-            for (int i = 0; i < a.length; i++) {
-                if (!a[i].getType().equals(b[i].getType())) {
-                    answer = false;
-                    break;
-                }
-            }
-            return answer;
-        }
-        return false;
+        return ParameterUtils.parametersEqual(a, b);
     }
 
     /**
@@ -1088,15 +1083,31 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     public MethodNode getGetterMethod(String getterName) {
+        return getGetterMethod(getterName, true);
+    }
+
+    public MethodNode getGetterMethod(String getterName, boolean searchSuperClasses) {
+        MethodNode getterMethod = null;
+        boolean booleanReturnOnly = getterName.startsWith("is");
         for (MethodNode method : getDeclaredMethods(getterName)) {
             if (getterName.equals(method.getName())
                     && ClassHelper.VOID_TYPE!=method.getReturnType()
-                    && method.getParameters().length == 0) {
-                return method;
+                    && method.getParameters().length == 0
+                    && (!booleanReturnOnly || ClassHelper.Boolean_TYPE.equals(ClassHelper.getWrapper(method.getReturnType())))) {
+                // GROOVY-7363: There can be multiple matches for a getter returning a generic parameter type, due to
+                // the generation of a bridge method. The real getter is really the non-bridge, non-synthetic one as it
+                // has the most specific and exact return type of the two. Picking the bridge method results in loss of
+                // type information, as it down-casts the return type to the lower bound of the generic parameter.
+                if (getterMethod == null || getterMethod.isSynthetic()) {
+                    getterMethod = method;
+                }
             }
         }
-        ClassNode parent = getSuperClass();
-        if (parent!=null) return parent.getGetterMethod(getterName);
+        if (getterMethod != null) return getterMethod;
+        if (searchSuperClasses) {
+            ClassNode parent = getSuperClass();
+            if (parent != null) return parent.getGetterMethod(getterName);
+        }
         return null;
     }
 
@@ -1155,52 +1166,51 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         if (isArray()) {
             return componentType.toString(showRedirect)+"[]";
         }
-        String ret = getName();
-        if (placeholder) ret = getUnresolvedName();
+        StringBuilder ret = new StringBuilder(getName());
+        if (placeholder) ret = new StringBuilder(getUnresolvedName());
         if (!placeholder && genericsTypes != null) {
-            ret += " <";
+            ret.append(" <");
             for (int i = 0; i < genericsTypes.length; i++) {
-                if (i != 0) ret += ", ";
+                if (i != 0) ret.append(", ");
                 GenericsType genericsType = genericsTypes[i];
-                ret += genericTypeAsString(genericsType, showRedirect);
+                ret.append(genericTypeAsString(genericsType));
             }
-            ret += ">";
+            ret.append(">");
         }
         if (redirect != null && showRedirect) {
-            ret += " -> " + redirect().toString();
+            ret.append(" -> ").append(redirect().toString());
         }
-        return ret;
+        return ret.toString();
     }
 
     /**
      * This exists to avoid a recursive definition of toString. The default toString
      * in GenericsType calls ClassNode.toString(), which calls GenericsType.toString(), etc. 
      * @param genericsType
-     * @param showRedirect
      * @return the string representing the generic type
      */
-    private String genericTypeAsString(GenericsType genericsType, boolean showRedirect) {
-        String ret = genericsType.getName();
+    private String genericTypeAsString(GenericsType genericsType) {
+        StringBuilder ret = new StringBuilder(genericsType.getName());
         if (genericsType.getUpperBounds() != null) {
-            ret += " extends ";
+            ret.append(" extends ");
             for (int i = 0; i < genericsType.getUpperBounds().length; i++) {
                 ClassNode classNode = genericsType.getUpperBounds()[i];
                 if (classNode.equals(this)) {
-                    ret += classNode.getName();
+                    ret.append(classNode.getName());
                 } else {
-                    ret += classNode.toString(showRedirect);
+                    ret.append(classNode.toString(false));
                 }
-                if (i + 1 < genericsType.getUpperBounds().length) ret += " & ";
+                if (i + 1 < genericsType.getUpperBounds().length) ret.append(" & ");
             }
-        } else if (genericsType.getLowerBound() !=null) {
+        } else if (genericsType.getLowerBound() != null) {
             ClassNode classNode = genericsType.getLowerBound();
             if (classNode.equals(this)) {
-                ret += " super " + classNode.getName();
+                ret.append(" super ").append(classNode.getName());
             } else {
-                ret += " super " + classNode;
+                ret.append(" super ").append(classNode.toString(false));
             }
         }
-        return ret;
+        return ret.toString();
     }
 
     /**
@@ -1299,17 +1309,17 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         } else if (arguments instanceof MapExpression) {
             count = 1;
         }
-        
+
         for (MethodNode method : getMethods(name)) {
             if(method.isStatic()) {
-                Parameter[] parameters = method.getParameters(); 
+                Parameter[] parameters = method.getParameters();
                 if (parameters.length == count) return true;
 
                 // handle varargs case
                 if (parameters.length > 0 && parameters[parameters.length - 1].getType().isArray()) {
                     if (count >= parameters.length - 1) return true;
                 }
-                
+
                 // handle parameters with default values
                 int nonDefaultParameters = 0;
                 for (Parameter parameter : parameters) {
@@ -1451,7 +1461,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     public void renameField(String oldName, String newName) {
         ClassNode r = redirect ();
         if (r.fieldIndex == null)
-            r.fieldIndex = new HashMap<String,FieldNode> ();
+            r.fieldIndex = new LinkedHashMap<String,FieldNode> ();
         final Map<String,FieldNode> index = r.fieldIndex;
         index.put(newName, index.remove(oldName));
     }
@@ -1459,7 +1469,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     public void removeField(String oldName) {
         ClassNode r = redirect ();
         if (r.fieldIndex == null)
-            r.fieldIndex = new HashMap<String,FieldNode> ();
+            r.fieldIndex = new LinkedHashMap<String,FieldNode> ();
         final Map<String,FieldNode> index = r.fieldIndex;
         r.fields.remove(index.get(oldName));
         index.remove(oldName);
@@ -1480,7 +1490,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         if(transformInstances == null){
             transformInstances = new EnumMap<CompilePhase, Map<Class <? extends ASTTransformation>, Set<ASTNode>>>(CompilePhase.class);
             for (CompilePhase phase : CompilePhase.values()) {
-                transformInstances.put(phase, new HashMap<Class <? extends ASTTransformation>, Set<ASTNode>>());
+                transformInstances.put(phase, new LinkedHashMap<Class <? extends ASTTransformation>, Set<ASTNode>>());
             }
         }
         return transformInstances;
