@@ -27,7 +27,23 @@ import org.codehaus.groovy.ast.FieldNode;
 import org.codehaus.groovy.ast.InnerClassNode;
 import org.codehaus.groovy.ast.MethodNode;
 import org.codehaus.groovy.ast.Parameter;
-import org.codehaus.groovy.ast.expr.*;
+import org.codehaus.groovy.ast.expr.ArgumentListExpression;
+import org.codehaus.groovy.ast.expr.ArrayExpression;
+import org.codehaus.groovy.ast.expr.BinaryExpression;
+import org.codehaus.groovy.ast.expr.BooleanExpression;
+import org.codehaus.groovy.ast.expr.ClassExpression;
+import org.codehaus.groovy.ast.expr.ConstantExpression;
+import org.codehaus.groovy.ast.expr.ConstructorCallExpression;
+import org.codehaus.groovy.ast.expr.DeclarationExpression;
+import org.codehaus.groovy.ast.expr.Expression;
+import org.codehaus.groovy.ast.expr.FieldExpression;
+import org.codehaus.groovy.ast.expr.ListExpression;
+import org.codehaus.groovy.ast.expr.MapEntryExpression;
+import org.codehaus.groovy.ast.expr.MapExpression;
+import org.codehaus.groovy.ast.expr.MethodCallExpression;
+import org.codehaus.groovy.ast.expr.SpreadExpression;
+import org.codehaus.groovy.ast.expr.StaticMethodCallExpression;
+import org.codehaus.groovy.ast.expr.VariableExpression;
 import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.ast.stmt.EmptyStatement;
 import org.codehaus.groovy.ast.stmt.ExpressionStatement;
@@ -90,7 +106,7 @@ public class EnumVisitor extends ClassCodeVisitorSupport {
         addInit(enumClass, minValue, maxValue, values, isAic);
     }
 
-    private void checkForAbstractMethods(ClassNode enumClass) {
+    private static void checkForAbstractMethods(ClassNode enumClass) {
         List<MethodNode> methods = enumClass.getMethods();
         for (MethodNode m : methods) {
             if (m.isAbstract()) {
@@ -101,7 +117,7 @@ public class EnumVisitor extends ClassCodeVisitorSupport {
         }
     }
 
-    private void addMethods(ClassNode enumClass, FieldNode values) {
+    private static void addMethods(ClassNode enumClass, FieldNode values) {
         List<MethodNode> methods = enumClass.getMethods();
 
         boolean hasNext = false;
@@ -361,7 +377,7 @@ public class EnumVisitor extends ClassCodeVisitorSupport {
                     }
                     args.addExpression(exp);
                 }
-                if (savedMapEntries.size() > 0) {
+                if (!savedMapEntries.isEmpty()) {
                     args.getExpressions().add(2, new MapExpression(savedMapEntries));
                 }
             }
@@ -419,7 +435,7 @@ public class EnumVisitor extends ClassCodeVisitorSupport {
         );
     }
 
-    private boolean isAnonymousInnerClass(ClassNode enumClass) {
+    private static boolean isAnonymousInnerClass(ClassNode enumClass) {
         if (!(enumClass instanceof EnumConstantClassNode)) return false;
         InnerClassNode ic = (InnerClassNode) enumClass;
         return ic.getVariableScope() == null;
