@@ -18,7 +18,24 @@
  */
 package org.codehaus.groovy.runtime;
 
-import groovy.lang.*;
+import groovy.lang.Binding;
+import groovy.lang.Closure;
+import groovy.lang.GString;
+import groovy.lang.GroovyInterceptable;
+import groovy.lang.GroovyObject;
+import groovy.lang.GroovyRuntimeException;
+import groovy.lang.GroovySystem;
+import groovy.lang.MetaClass;
+import groovy.lang.MetaClassRegistry;
+import groovy.lang.MissingMethodException;
+import groovy.lang.MissingPropertyException;
+import groovy.lang.Range;
+import groovy.lang.Script;
+import groovy.lang.SpreadMap;
+import groovy.lang.SpreadMapEvaluatingException;
+import groovy.lang.Tuple;
+import groovy.lang.Writable;
+import org.codehaus.groovy.reflection.ClassInfo;
 import org.codehaus.groovy.runtime.metaclass.MetaClassRegistryImpl;
 import org.codehaus.groovy.runtime.metaclass.MissingMethodExecutionFailed;
 import org.codehaus.groovy.runtime.powerassert.PowerAssertionError;
@@ -66,6 +83,7 @@ public class InvokerHelper {
 
     public static void removeClass(Class clazz) {
         metaRegistry.removeMetaClass(clazz);
+        ClassInfo.remove(clazz);
         Introspector.flushFromCaches(clazz);
     }
 
@@ -424,14 +442,7 @@ public class InvokerHelper {
         } else {
             try {
                 if (Script.class.isAssignableFrom(scriptClass)) {
-                    try {
-                        Constructor constructor = scriptClass.getConstructor(Binding.class);
-                        script = (Script) constructor.newInstance(context);
-                    } catch (NoSuchMethodException e) {
-                        // Fallback for non-standard "Script" classes.
-                        script = (Script) scriptClass.newInstance();
-                        script.setBinding(context);
-                    }
+                    script = newScript(scriptClass, context);
                 } else {
                     final GroovyObject object = (GroovyObject) scriptClass.newInstance();
                     // it could just be a class, so let's wrap it in a Script
@@ -469,6 +480,19 @@ public class InvokerHelper {
         return script;
     }
 
+    public static Script newScript(Class scriptClass, Binding context) throws InstantiationException, IllegalAccessException, InvocationTargetException {
+        Script script;
+        try {
+            Constructor constructor = scriptClass.getConstructor(Binding.class);
+            script = (Script) constructor.newInstance(context);
+        } catch (NoSuchMethodException e) {
+            // Fallback for non-standard "Script" classes.
+            script = (Script) scriptClass.newInstance();
+            script.setBinding(context);
+        }
+        return script;
+    }
+
     /**
      * Sets the properties on the given object
      */
@@ -616,13 +640,7 @@ public class InvokerHelper {
         }
         if (arguments instanceof String) {
             if (verbose) {
-                String arg = ((String) arguments)
-                        // must replace backslashes first, as the other replacements add backslashes not to be escaped
-                        .replace("\\", "\\\\")      // backslash
-                        .replace("\n", "\\n")    // line feed
-                        .replaceAll("\\r", "\\\\r")      // carriage return
-                        .replaceAll("\\t", "\\\\t")      // tab
-                        .replaceAll("\\f", "\\\\f")      // form feed
+                String arg = escapeBackslashes((String) arguments)
                         .replaceAll("'", "\\\\'");    // single quotation mark
                 return "\'" + arg + "\'";
             } else {
@@ -634,6 +652,16 @@ public class InvokerHelper {
         return arguments.toString();
     }
 
+    public static String escapeBackslashes(String orig) {
+        // must replace backslashes first, as the other replacements add backslashes not to be escaped
+        return orig
+                .replace("\\", "\\\\")           // backslash
+                .replace("\n", "\\n")            // line feed
+                .replaceAll("\\r", "\\\\r")      // carriage return
+                .replaceAll("\\t", "\\\\t")      // tab
+                .replaceAll("\\f", "\\\\f");     // form feed
+    }
+
     private static String formatMap(Map map, boolean verbose, int maxSize) {
         if (map.isEmpty()) {
             return "[:]";
