@@ -26,6 +26,7 @@ import javax.swing.text.Position;
 import javax.swing.text.Segment;
 import javax.swing.text.Style;
 import javax.swing.text.StyleContext;
+import java.io.Serializable;
 import java.nio.CharBuffer;
 import java.util.ArrayList;
 import java.util.Comparator;
@@ -71,13 +72,13 @@ public class StructuredSyntaxDocumentFilter extends DocumentFilter {
         String checking = regexp.replaceAll("\\\\\\(", "X").replaceAll("\\(\\?", "X");
         int checked = checking.indexOf('(');
         if (checked > -1) {
-            String msg = "Only non-capturing groups allowed:\r\n" +
-                         regexp + "\r\n";
+            StringBuilder msg = new StringBuilder("Only non-capturing groups allowed:\r\n" +
+                    regexp + "\r\n");
             for (int i = 0; i < checked; i++) {
-                msg += " ";
+                msg.append(" ");
             }
-            msg += "^";
-            throw new IllegalArgumentException(msg);
+            msg.append("^");
+            throw new IllegalArgumentException(msg.toString());
         }
     }
     
@@ -269,7 +270,11 @@ public class StructuredSyntaxDocumentFilter extends DocumentFilter {
      */    
     public void replace(DocumentFilter.FilterBypass fb, int offset, 
                         int length, String text, AttributeSet attrs)
-        throws BadLocationException {
+        throws BadLocationException
+    {
+        // text might be null and indicates no replacement text
+        if (text == null) text = "";
+        
         // remove problem meta characters returns
         text = replaceMetaCharacters(text);
         
@@ -309,10 +314,10 @@ public class StructuredSyntaxDocumentFilter extends DocumentFilter {
         }
     
         private String buildRegexp(String[] regexps) {
-            String regexp = "";
+            StringBuilder regexp = new StringBuilder();
 
             for (int i = 0; i < regexps.length; i++) {
-                regexp += "|" + regexps[i];
+                regexp.append("|").append(regexps[i]);
             }
 
             // ensure leading '|' is removed
@@ -329,14 +334,14 @@ public class StructuredSyntaxDocumentFilter extends DocumentFilter {
             groupList.add(null);
             
             Iterator iter = styleMap.keySet().iterator();
-            String regexp = "";
+            StringBuilder regexp = new StringBuilder();
             while (iter.hasNext()) {
                 String nextRegexp = (String)iter.next();
-                regexp += "|(" + nextRegexp + ")";
+                regexp.append("|(").append(nextRegexp).append(")");
                 // have to compile regexp first so that it will match
                 groupList.add(Pattern.compile(nextRegexp).pattern());
             }
-            if (!regexp.equals("")) {
+            if (!regexp.toString().equals("")) {
                 matcher = Pattern.compile(regexp.substring(1)).matcher("");
                 
                 iter = children.values().iterator();
@@ -553,8 +558,10 @@ public class StructuredSyntaxDocumentFilter extends DocumentFilter {
         
     }
 
-    private static class MLComparator implements Comparator {
-        
+    private static class MLComparator implements Comparator, Serializable {
+
+        private static final long serialVersionUID = -4210196728719411217L;
+
         public int compare(Object obj, Object obj1) {
             return valueOf(obj) - valueOf(obj1);
         }
