@@ -30,7 +30,18 @@ import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.net.URL;
 import java.text.SimpleDateFormat;
-import java.util.*;
+import java.util.Arrays;
+import java.util.Calendar;
+import java.util.Collections;
+import java.util.Date;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Locale;
+import java.util.Map;
+import java.util.TimeZone;
+import java.util.UUID;
 
 /**
  * Class responsible for the actual String serialization of the possible values of a JSON structure.
@@ -305,10 +316,7 @@ public class JsonOutput {
             } else if (Enum.class.isAssignableFrom(objectClass)) {
                 buffer.addQuoted(((Enum<?>) object).name());
             }else if (File.class.isAssignableFrom(objectClass)){
-                Map<?, ?> properties = DefaultGroovyMethods.getProperties(object);
-                properties.remove("class");
-                properties.remove("declaringClass");
-                properties.remove("metaClass");
+                Map<?, ?> properties = getObjectProperties(object);
                 //Clean up all recursive references to File objects
                 Iterator<? extends Map.Entry<?, ?>> iterator = properties.entrySet().iterator();
                 while(iterator.hasNext()){
@@ -320,15 +328,20 @@ public class JsonOutput {
 
                 writeMap(properties, buffer);
             } else {
-                Map<?, ?> properties = DefaultGroovyMethods.getProperties(object);
-                properties.remove("class");
-                properties.remove("declaringClass");
-                properties.remove("metaClass");
+                Map<?, ?> properties = getObjectProperties(object);
                 writeMap(properties, buffer);
             }
         }
     }
 
+    private static Map<?, ?> getObjectProperties(Object object) {
+        Map<?, ?> properties = DefaultGroovyMethods.getProperties(object);
+        properties.remove("class");
+        properties.remove("declaringClass");
+        properties.remove("metaClass");
+        return properties;
+    }
+
 
     /**
      * Serializes any char sequence and writes it into specified buffer.
@@ -490,8 +503,8 @@ public class JsonOutput {
      */
     public static String prettyPrint(String jsonPayload) {
         int indentSize = 0;
-        // Just a guess that the pretty view will take a 20 percent more than original.
-        final CharBuf output = CharBuf.create((int) (jsonPayload.length() * 0.2));
+        // Just a guess that the pretty view will take 20 percent more than original.
+        final CharBuf output = CharBuf.create((int) (jsonPayload.length() * 1.2));
 
         JsonLexer lexer = new JsonLexer(new StringReader(jsonPayload));
         // Will store already created indents.
